
using System.IO;
using UnityEngine;
using Zepeto;

public class MakeupConverter {

	public static Texture2D CullingMakeupTexture(Texture2D texture) {
		if (texture == null) return null;
		
		var maskPixels = PrepareMaskPixels(texture.width, texture.height);

		var pixels = texture.GetPixels();
		for (int i = 0; i < texture.height; ++i) {
			for (int j = 0; j < texture.width; ++j) {
				if (maskPixels[i * texture.width + j].Equals(Color.black)) {
					pixels[i * texture.width + j] = Color.clear;
				}
			}
		}
		
		var result = new Texture2D(texture.width, texture.height);
		result.SetPixels(pixels);
		result.Apply();

		return result;
	}

	private static Color[] PrepareMaskPixels(int width, int height) {
		var skinMesh = new Mesh();
		skinMesh.vertices = new[] {new Vector3(2 / 8f * -4f, 1, 0), new Vector3(2 / 8f * -3f, 1, 0), new Vector3(2 / 8f * -2f, 1, 0), new Vector3(2 / 8f * -1f, 1, 0), new Vector3(2 / 8f * 0f, 1, 0), new Vector3(2 / 8f * 1f, 1, 0), new Vector3(2 / 8f * 2f, 1, 0), new Vector3(2 / 8f * 3f, 1, 0), new Vector3(2 / 8f * 4f, 1, 0), new Vector3(2 / 6f * -3f, -1, 0), new Vector3(2 / 6f * -2f, -1, 0), new Vector3(2 / 6f * -1f, -1, 0), new Vector3(2 / 6f * 0f, -1, 0), new Vector3(2 / 6f * 1f, -1, 0), new Vector3(2 / 6f * 2f, -1, 0), new Vector3(2 / 6f * 3f, -1, 0)};
		skinMesh.triangles = new[] {0, 1, 9, 1, 10, 9, 1, 2, 10, 2, 11, 10, 2, 3, 11, 3, 12, 11, 3, 4, 12, 4, 5, 12, 5, 13, 12, 5, 6, 13, 6, 14, 13, 6, 7, 14, 7, 15, 14, 7, 8, 15};
		skinMesh.uv = new[] {new Vector2(1 / 8f * 0f, 1), new Vector2(1 / 8f * 1f, 1), new Vector2(1 / 8f * 2f, 1), new Vector2(1 / 8f * 3f, 1), new Vector2(1 / 8f * 4f, 1), new Vector2(1 / 8f * 5f, 1), new Vector2(1 / 8f * 6f, 1), new Vector2(1 / 8f * 7f, 1), new Vector2(1 / 8f * 8f, 1), new Vector2(1 / 6f * 0f, 0f), new Vector2(1 / 6f * 1f, 0f), new Vector2(1 / 6f * 2f, 0f), new Vector2(1 / 6f * 3f, 0f), new Vector2(1 / 6f * 4f, 0f), new Vector2(1 / 6f * 5f, 0f), new Vector2(1 / 6f * 6f, 0f)};
		skinMesh.vertices = new[] {new Vector3(-241 / 256f, 9 / 256f, 0), new Vector3(-191 / 256f, 30 / 256f, 0), new Vector3(-156 / 256f, 138 / 256f, 0), new Vector3(-86 / 256f, 200 / 256f, 0), new Vector3(0 / 256f, 227 / 256f, 0), new Vector3(86 / 256f, 200 / 256f, 0), new Vector3(156 / 256f, 138 / 256f, 0), new Vector3(191 / 256f, 30 / 256f, 0), new Vector3(241 / 256f, 9 / 256f, 0), new Vector3(-217 / 256f, -70 / 256f, 0), new Vector3(-172 / 256f, -73 / 256f, 0), new Vector3(-136 / 256f, -126 / 256f, 0), new Vector3(0 / 256f, -176 / 256f, 0), new Vector3(136 / 256f, -126 / 256f, 0), new Vector3(172 / 256f, -73 / 256f, 0), new Vector3(217 / 256f, -70 / 256f, 0)};
		
		var result = Draw(skinMesh);
		Object.DestroyImmediate(skinMesh);

		var scaledPixels = new Color[width * height];
		for (int i = 0; i < height; ++i) {
			for (int j = 0; j < width; ++j) {
				var newColor = result.GetPixelBilinear(j / (float)width, i / (float)height);
				scaledPixels[i * width + j] = newColor;
			}
		}
		Object.DestroyImmediate(result);

		return scaledPixels;
	}
	
	private static Texture2D Draw(Mesh skinMesh) {
		var material = new Material(ZepetoInitializer.TextureMixerShader);
		material.color = Color.white;
		
		var _blendModePropertyId = Shader.PropertyToID("_BlendMode");
		var _alphaPropertyId = Shader.PropertyToID("_Alpha");
		var _backgroundTexPropertyId = Shader.PropertyToID("_BackgroundTex");
		
		material.mainTexture = Texture2D.whiteTexture;
		material.SetInt(_blendModePropertyId, 0);
		material.SetFloat(_alphaPropertyId, 1);
		material.SetTexture(_backgroundTexPropertyId, null);
		
		
		var old = RenderTexture.active;
		var rt = new RenderTexture(512, 512, 0);
		RenderTexture.active = rt;
			
		GL.Clear(true, true, Color.black);

		if (material.SetPass(0)) {
			Graphics.DrawMeshNow(skinMesh, Matrix4x4.identity);
		}
		
		var output = new Texture2D(rt.width, rt.height);
		output.ReadPixels(new Rect(0, 0, rt.width, rt.height), 0, 0);
		output.Apply();
		
		RenderTexture.active = old;
		
		Object.DestroyImmediate(material);
		Object.DestroyImmediate(rt);

		return output;
	}
}
